@@ -1,13 +1,6 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+# Copyright (c) 2017-2024 trollimage developers
 #
-# Copyright (c) 2017-2018
-#
-# Author(s):
-#
-#   Martin Raspaud <martin.raspaud@smhi.se>
-#   Adam Dybbroe <adam.dybbroe@smhi.se>
-#   Esben S. Nielsen <esn@dmi.dk>
+# This file is part of trollimage.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -830,20 +823,25 @@ class XRImage:
             return data.attrs.get('_FillValue')
         return None
 
-    def _scale_and_replace_fill_value(self, data, input_fill_value, fill_value, dtype):
+    def _scale_and_replace_fill_value(self, data, input_fill_value, fill_value,
+                                      dtype, scale, replace_fill_value):
         # scale float data to the proper dtype
         # this method doesn't cast yet so that we can keep track of NULL values
-        data = self._scale_to_dtype(data, dtype, fill_value)
-        data = self._replace_fill_value(data, input_fill_value, fill_value, dtype)
+        if scale:
+            data = self._scale_to_dtype(data, dtype, fill_value)
+        if replace_fill_value:
+            data = self._replace_fill_value(data, input_fill_value, fill_value, dtype)
         return data
 
-    def _scale_alpha_or_fill_data(self, data, fill_value, dtype):
+    def _scale_alpha_or_fill_data(self, data, fill_value, dtype, scale, alpha, replace_fill_value):
         input_fill_value = self._get_input_fill_value(data)
-        needs_alpha = fill_value is None and not self.mode.endswith('A')
+        needs_alpha = alpha and fill_value is None and not self.mode.endswith('A')
         if needs_alpha:
             # We don't have a fill value or an alpha, let's add an alpha
             return self._add_alpha_and_scale(data, input_fill_value, dtype)
-        return self._scale_and_replace_fill_value(data, input_fill_value, fill_value, dtype)
+        return self._scale_and_replace_fill_value(data, input_fill_value,
+                                                  fill_value, dtype, scale,
+                                                  replace_fill_value)
 
     def finalize(self, fill_value=None, dtype=np.uint8, keep_palette=False):
         """Finalize the image to be written to an output file.
@@ -908,8 +906,11 @@ class XRImage:
             pass
         with xr.set_options(keep_attrs=True):
             attrs = final_data.attrs
-            if not keep_palette:
-                final_data = self._scale_alpha_or_fill_data(final_data, fill_value, dtype)
+            final_data = self._scale_alpha_or_fill_data(
+                    final_data, fill_value, dtype,
+                    scale=not keep_palette,
+                    alpha=not keep_palette,
+                    replace_fill_value=True)
             final_data = final_data.astype(dtype)
             final_data.attrs = attrs
 
@@ -1132,7 +1133,6 @@ class XRImage:
         if val is None:
             non_band_dims = tuple(x for x in self.data.dims if x != 'bands')
             val = getattr(self.data, kind)(dim=non_band_dims)
-
         if isinstance(val, (list, tuple)):
             val = self.xrify_tuples(val)
 
@@ -1302,7 +1302,7 @@ class XRImage:
         logger.debug("Applying invert with parameters %s", str(invert))
         if isinstance(invert, (tuple, list)):
             invert = self.xrify_tuples(invert)
-            offset = invert.astype(int)
+            offset = invert.astype(self.data.dtype)
             scale = (-1) ** offset
         elif invert:
             offset = 1
@@ -1428,6 +1428,9 @@ class XRImage:
         To (directly) get an image in mode "RGB" or "RGBA", use
         :meth:`~XRImage.colorize`.
 
+        Invalid data (NaN) are sorted into the final bin and the ``_FillValue``
+        attributed for the resulting image is set to the corresponding value.
+
         Args:
             colormap (:class:`~trollimage.colormap.Colormap`):
                 Colormap to be applied to the image.
@@ -1481,8 +1484,10 @@ class XRImage:
             mode = "PA"
             new_data = da.concatenate([new_data, self.data.sel(bands=['A'])], axis=0)
 
+        old_dtype = self.data.dtype
         self.data.data = new_data
         self.data.coords['bands'] = list(mode)
+        self._set_new_fill_value_after_palettize(colormap, old_dtype)
         # See docstring notes above for how scale/offset should be used
         scale_factor, offset = self._get_colormap_scale_offset(colormap)
         self.data.attrs.setdefault('enhancement_history', []).append({
@@ -1499,6 +1504,28 @@ class XRImage:
         offset = -cmap_min * scale_factor
         return scale_factor, offset
 
+    def _set_new_fill_value_after_palettize(self, colormap, old_dtype):
+        """Set new fill value after palettizing."""
+        # OK: float without fill value or fill value nan
+        # OK: int without fill value or fill value to max value
+        if ((np.issubdtype(old_dtype, np.inexact) and
+             np.isnan(self.data.attrs.get("_FillValue", np.nan))) or
+            (np.issubdtype(old_dtype, np.integer) and
+             self.data.attrs.get("_FillValue", np.iinfo(old_dtype).max) == np.iinfo(old_dtype).max)):
+            self.data.attrs["_FillValue"] = colormap.values.shape[0]-1
+        # not OK: float or int with different fill value
+        elif "_FillValue" in self.data.attrs:
+            warnings.warn(
+                f"Palettizing {old_dtype.name:s} data with the _FillValue attribute set to "
+                f"{self.data.attrs['_FillValue']!s}, "
+                "but palettize is not generally fill value aware (masked data "
+                "will be correctly palettized only for float with NaN or for "
+                "ints with fill value set to dtype max.",
+                UserWarning,
+                stacklevel=3)
+        # else: non-numeric data, probably doesn't work at all and will fail
+        # elsewhere anyway
+
     def blend(self, src):
         r"""Alpha blend *src* on top of the current image.
 
